#!/usr/bin/perl
package BugExtractor;
use strict;
use EmailExtractor;

sub new {
    my ($type,%options) = @_;
    my $type = shift;
    return bless(\%options,$type);
}


sub mk_printer { 
    my ($fd) = @_;
    sub { my $ex = shift; print $fd $ex->csv_line(@_) }
};

# To get the printers
# my ($bug_entity, $bug_field, $bugs, $bug_people) = $bug->make_printers();
sub make_printers {
    my ($self) = @_;
    if ($self->{_made_printers}) {
        warn "Printers already made!";
        return @{$self->{printers}};
    }
    my ($f_bug_people, $f_bugs, $f_bug_fields, $f_bug_entity);
    open($f_bug_people, ">", "query1.bug_people");
    open($f_bugs, ">", "query1.bugs");
    open($f_bug_fields, ">", "query1.bug_fields");
    open($f_bug_entity, ">", "query1.bug_entity");
    my $bug_entity = mk_printer( $f_bug_entity );
    my $bug_field  = mk_printer( $f_bug_fields );
    my $bugs       = mk_printer( $f_bugs       );
    my $bug_people = mk_printer( $f_bug_people );

    $self->{_made_printers} = 1;

    $self->{printers} = [$bug_entity, $bug_field, $bugs, $bug_people ];
    return @{$self->{printers}};
}

sub close_files {
    my ($self) = @_;
    die "Didn't open files!" unless $self->{_made_printers};
    close($self->{f_bug_people});
    close($self->{f_bugs});
    close($self->{f_bug_fields});
    close($self->{f_bug_entity});
    delete $self->{_made_printers};
}

# Handler looks like this
# my $handler = sub {
#    my ($ex, $h, $email_map, $errors, $msg) = @_;
# };
# Note: the handler does the DB writing by hand!

sub argv_parse {
    my ($self,$handler, @ARGV) = @_;
    die "Didn't mk_printers" unless $self->{_made_printers};

    my ($bug_entity, $bug_field, $bugs, $bug_people) = @{$self->{printers}};

    my $ex = EmailExtractor->new();

    #bugid | entityid | sutime | author | sdate | cutime | cdate | fields | text

    if (@ARGV) {
        my @errors = $ex->parse_file( $_, $handler) foreach @ARGV;
        handle_errors(@errors);
    } else {
        my @errors = $ex->parse_file( undef, $handler);
        handle_errors(@errors);
    }
}

sub trim {
    my ($a) = @_;
    $a =~ s/^\s*//;
    $a =~ s/\s*$//;
    return $a;
}

sub handle_errors {
     open(my $fd, ">>", "error.log");
     print $fd $_.$/ foreach @_;
     close($fd);
}



1;
