#!/usr/bin/perl
use strict;
use EmailExtractor;

# This extractor extracts emails mails csv files out of them

#ok we're going to produce query1.mailinglist
#                          query1.emails
#                          query1.email_map to list id

my $q1mailinglist;
my $q1emails;
my $q1email_map;
my $q1emailevent;
my $errorlog;
open($q1mailinglist,">","query1.mailinglist");
open($q1emails,">","query1.emails");
open($q1email_map,">","query1.email_map");
open($q1emailevent,">","query1.emailevent");
open($errorlog,">","error.log");

my @mlcolumns = qw(list id date daterev timerev utime from_name from_email to entities subject body quote0 quote1 quote2 quoterest sig nbodyemails bodyemails);
my @emcolumn = qw(address name count);
my @mapcolumn = qw(address list id type); #FROM TO BODY
my @evcolumn = qw(list id date daterev timerev utime subject);

my $ex = EmailExtractor->new();

warn "Printing headers!";
print $q1mailinglist $ex->header_string(@mlcolumns);
print $q1emails      $ex->header_string(@emcolumn);
print $q1email_map   $ex->header_string(@mapcolumn);
print $q1emailevent  $ex->header_string(@evcolumn);

if (@ARGV) {
    my @errors = $ex->parse_file( $_, \&handler) foreach @ARGV;
    handle_errors(@errors);
} else {
    my @errors = $ex->parse_file( undef, \&handler);
    handle_errors(@errors);
}

dump_email_map($ex->email_map);

close($q1emailevent);
close($q1email_map);
close($q1emails);
close($q1mailinglist);
close($errorlog);

sub handler {
    my ($ex, $h, $email_map, $errors) = @_;
    print $q1mailinglist  $ex->csv_line( map { $h->{$_} } @mlcolumns );
    print $q1emailevent   $ex->csv_line( map { $h->{$_} } @evcolumn  );
        # now we take all the seen emails and print them out
        {
            print $q1email_map $ex->csv_line( $h->{femail}, $h->{listname}, $h->{id}, "FROM");
            foreach my $to (@{$h->{tos}}) {
                print $q1email_map $ex->csv_line($to, $h->{listname}, $h->{id}, "TO");
            }
            foreach my $mention (sort (keys %{$h->{emails_hash}})) {
                print $q1email_map $ex->csv_line( $mention, $h->{listname}, $h->{id}, "BODY");
            }
        }
}

sub dump_email_map {
    my ($email_map) = @_;
    while (my ($email,$names) = each %{$email_map}) {
        while (my ($name,$count) = each %{$names}) {
            print $q1emails (join("|",map {EmailExtractor::escape($_)} ($email,$name,$count)),$/);
        }
    }
}


sub handle_errors {
    print $errorlog $_.$/ foreach @_;
}
