#!/usr/bin/perl
use strict;
use EmailExtractor;
use BugExtractor;

my $bug = BugExtractor->new();

#get the printers
my ($bug_entity, $bug_field, $bugs, $bug_people) = $bug->make_printers();


my $handler = sub {
    my ($ex, $h, $email_map, $errors, $msg) = @_;

    my $header = $msg->header;
    my $pr_version = $header->{'x-send-pr-version'};
    my $is_gnats = (length($pr_version) > 0)?1:0;
    my $entity_id = $h->{id};
    my $body = $h->{body};
    my $date = $msg->{date};
    my $subject = $h->{subject};
    if ($is_gnats) {
        my ($gnats_hash, $keys, $fields) = parse_gnats($h->{body});
        my $date = $gnats_hash->{'Arrival-Date'} || $msg->{date};
        my $utime = EmailExtractor::get_utime_from_date($date);
        my $ymd = EmailExtractor::ymd_from_utime($utime);
        my $cdate = $gnats_hash->{'Arrival-Date'} || "";
        my $cutime = EmailExtractor::get_utime_from_date($cdate);
        my $cymd = EmailExtractor::ymd_from_utime($cutime);

        my $author = $gnats_hash->{Originator} || $msg->{fname};
        #bugid | entityid | sutime | author | sdate | cutime | cdate | fields | text
        my $bug_id = $gnats_hash->{Number};
        my @o = ();
        unless ($bug_id) {
            warn "Lacking a bug_id!";
            push @$errors, "Lacking a bug id! $entity_id";
            return;
        }

        # FreeBSD Normalized bug_id
        $bug_id = $h->{basename}."/".$bug_id;

        while (my ($key,$val) = each %$gnats_hash) {
            #bugid | entityid | utime | field | value
            &$bug_field($ex, $bug_id, $entity_id, $utime, $key, $val);
        }
        while (my ($key,$val) = each %$fields) {
            push @o, "$key: $val";
        }
        my $field = join($/,@o);

        &$bug_entity($ex, $bug_id, $entity_id, $utime, $author, $ymd,
                        $cutime, $cymd, $field , $body);
        # bugid | utime | first_author | text
        &$bugs($ex, $bug_id, $utime, $author, $body);
        # bugpeople
        # bugid_| author
        foreach my $key ($gnats_hash->{Originator}, keys %{$h->{emails_hash}}) {
            &$bug_people($ex, $bug_id, $key );
        }
    } else {
        #warn "A non-gnats email found!";
        # this is likely a reply email
        my ($gnats_hash, $keys, $fields) = parse_gnats($h->{body});
        #delete $gnats_hash->{head};
        #delete $fields->{head};
        
        my $bug_id = $gnats_hash->{Number} || "";
        unless ($bug_id) {
            my $x = $h->{subject} || $h->{cc};
            #does it match
            # Subject: Re: kern/1256: ZNYX 314 mysterously looses packets 
            if ($x =~ /Re:\s*[^\s]+\/(\d+)\:\s(.*)$/i) {
                $bug_id = $1;
            }
        }
        #warn "With bugid $bug_id";
        my $utime = $h->{utime};
        my $author = $h->{fname};
        my $ymd = $h->{ymd};
        warn "No bugid! $subject, $entity_id, $author, $date" unless $bug_id;
        if ($bug_id) {
            $bug_id = $h->{basename}."/".$bug_id;

            &$bug_entity($ex, $bug_id, $entity_id, $utime, $author, $ymd,
                        $utime, $ymd, "" , $body);
            foreach my $key (qw(Originator), keys %{$h->{emails_hash}}) {
                &$bug_people($ex, $bug_id, $key );
            }
        }
    }
};
$bug->argv_parse($handler, @ARGV);

$bug->close_files();

# $5.45 
# $12.30 



sub parse_gnats {
    my ($body) = @_;
    my @lines = split(/(\r|\n)/ , $body);
    my $key = "head";
    my %h = ();
    my @keyorder = ();
    my %singles = ();
    foreach my $line (@lines) {
        # if line matches
        # >Number:         365
        # >Category:       kern
        # >Synopsis:       if_fddisubr.c doesn't compile (it never has...?!?)
        # >Confidential:   no
        # >Severity:       serious
        # >Priority:       medium
        # >Responsible:    freebsd-bugs (FreeBSD bugs mailing list)
        # >State:          closed
        # regex is non-whitespace-riddenname followed by colon
        #  followed by whitespace followed by content buffered by whitespace
        #   at least 1 non whitespace character
        if ($line =~ /^>([^:\s]+)\:\s*([^\s].*)\s*$/) {
            my $newkey = $1;
            my $value = $2;
            $h{$newkey} = $value;
            $singles{$newkey} = $value;
            $key = $newkey;
            push @keyorder, $key;
        # elsif line matches
        # >Quarter:
        } elsif ($line =~ /^>([^:\s]+)\:\s*$/) {
            $key = $1;
            $h{$key} = "" unless $h{$key};
            push @keyorder, $key;
        } else {
            $h{$key} .= $line . $/;
        }
    }
    foreach my $key (keys %h) {
        $h{$key} = BugExtractor::trim($h{$key});
    }
    return (\%h,\@keyorder, \%singles);
}

