#!/usr/bin/perl
use strict;
use EmailExtractor;
use BugExtractor;

use constant {
    HEAD => 1,
    HEADERS => 2,
    CDATA => 3,
};

my $bug_head_count = 0;
my $bug_count = 0;
my $other_count = 0;
my $email_count = 0;
my $bug = BugExtractor->new();

#get the printers
my ($bug_entity, $bug_field, $bugs, $bug_people) = $bug->make_printers();

my $handler = sub {
    my ($ex, $h, $email_map, $errors, $msg) = @_;
    $email_count++;
    my $header = $msg->header;
    my $entity_id = $h->{id};
    my $body = $h->{body};
    my $date = $msg->{date} || $h->{date};
    my $subject = $h->{subject};
    my ($bug_number, $bug_subject) = ($subject =~ /BUG\s*#(\d+): (.*)$/i);
    my $bug_id = $bug_number;
    my ($is_bug_head) = (($subject =~ /^\s*BUG/i) && $bug_number);
    my $is_bug = $bug_number > 0;
    if ($is_bug_head) {
        $bug_head_count++;
        my %bug = parse_postgresql($body);
        my $utime = EmailExtractor::get_utime_from_date($date);
        my $cdate = EmailExtractor::ymd_from_utime($utime);
        my $ymddate = $cdate;
        my $cutime = $utime;
        my $author = $bug{header}->{'Logged by'} || $msg->{fname};
        my $bug_id = $bug_number;
        die "No bugnumber in is_bug_head??" unless $bug_id;
        my @o = ();
        while (my ($key,$val) = each %{$bug{header}}) {
            &$bug_field($ex, $bug_id, $entity_id, $utime, $key, $val);
            push @o, "$key: $val";
        }
        my $field = join($/,@o);
        &$bug_entity($ex, $bug_id, $entity_id, $utime, $author, $ymddate,
                        $cutime, $cdate, $field , $body);
        # bugid | utime | first_author | text
        &$bugs($ex, $bug_id, $utime, $author, $body);
        # bugpeople
        # bugid_| author


        foreach my $key ($bug{header}->{'Email address'}, keys %{$h->{emails_hash}}) {
            &$bug_people($ex, $bug_id, $key );
        }
    } elsif ($is_bug) {
        $bug_count++;
        my $utime = $h->{utime};
        my $author = $h->{fname};     
        &$bug_entity($ex, $bug_id, $entity_id, $utime, $author, $h->{ymd},
                     $utime, $h->{ymd}, "" , $body);
        foreach my $key (qw(Originator), keys %{$h->{emails_hash}}) {
            &$bug_people($ex, $bug_id, $key );
        }
    } else {
        # A non bug email
        $other_count++;
        my $warning = "Email not a bug?: $subject";
        warn $warning;
        push @$errors, $warning;
    }
};

$bug->argv_parse($handler, @ARGV);

BugExtractor::handle_errors(<<EOM);
bug_head_count $bug_head_count 
bug_count      $bug_count      
other_count    $other_count    
email_count    $email_count    
EOM
$bug->close_files();

# Parse this:
# 
# The following bug has been logged online:
# 
# Bug reference:      4632
# Logged by:          Juan Moreno
# Email address:      juan@petlifestyleonline.com
# PostgreSQL version: 7.4.19
# Operating system:   Debian 4
# Description:        Duplicate
# Details: 
# 

sub parse_postgresql {
    my ($body) = @_;
    my $STATE = HEAD;
    my %h = ( header => {}, cdata => []);
    my $lastkey = undef;
    foreach my $line (split($/,$body)) {
        if (($STATE == HEAD || $STATE == HEADERS) && $line =~ /^\s*The following bug has been logged online:/) {
            #nothing
            $STATE = HEADERS;
        } elsif ($STATE == HEADERS && $line =~ /^Details:/) {
            $STATE = CDATA;
        } elsif ($STATE == HEADERS && $line =~ /^([^:]+):\s*(.*)\s*$/) {
            my ($key, $val) = ($1, $2);
            $h{header}->{$key} = BugExtractor::trim($val);
            $lastkey = $key;
        # just checking for missed header..
        } elsif ($STATE == HEAD && $line =~ /^([^:]+):\s*(.*)\s*$/) {
            warn "This line looks like a header: $line";
            push @{$h{head}}, $line;
        } elsif ($STATE == HEADERS && $line =~ /^\s*$/) {
            #DO NOTHING
        } elsif ($STATE == HEADERS) { # we got here because it is a continuation
            die "lastkey was not set [$line] [$lastkey] $/ [$body]" unless (defined $lastkey);
            $h{header}->{$lastkey} .= " ".$line;
        } elsif ($STATE == CDATA) {
            push @{$h{cdata}}, $line;
        } elsif ($line =~ /^\s*$/) {
            # DO NOTHING
        } elsif ($STATE == HEAD) {
            push @{$h{head}}, $line;
        } else {
            die "parse_postgresql: I did not plan to be here! [$line]";
        }
    }
    # in this case it might be an old style on
    if ($STATE == HEAD) {
        $h{cdata} = $h{head};
    }
    return %h
}



